
/*
 *	This include file (gpl.h) contains typedefs and defines for
 *	a GatherPlot object.
 */

#ifndef gpl_defs_
#define gpl_defs_

#undef	X_DEBUG

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <X11/Xlib.h>
#include <X11/Xresource.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>
#include <ctype.h>
#include <string.h>
#include <math.h>

#include "ol.h"
#include "tks.h"
#include "stock.h"

typedef struct box_ {
	Display *display;
	Window window;
	GC gc;
	int line_mode;
	int x;
	int y;
	int w;
	int h;
	int on;
} Box;


typedef struct gpl_trace {	       /* GPLTrace structure used in
				        * get_trace callback for passing
				        * trace info back to GatherPlot
				        * object. */
    char           *id_string;	       /* Station-Channel id string. */
    char           *units;	       /* Units string. */
    float           uvar;	       /* Uvar value. */
    int             nsegs;	       /* Number of segments. */
    double         *tstart;	       /* Nsegs start times. */
    double         *dt;		       /* Nsegs time increments. */
    int            *nsamps;	       /* Nsegs numbers of samples. */
    float         **data;	       /* Nsegs pointers to data samples. */
}               GPLTrace;

typedef struct gpl_grid {	       /* GPLGrid structure used in get_grid
				        * callback for passing grid info back
				        * to GatherPlot object. */
    double          t0;		       /* Start time. */
    int             nt;		       /* No. of time values */
    double          dt;		       /* time increment. */
    double          y0;		       /* Start Y-value. */
    int             ny;		       /* No. of Y-values */
    double          dy;		       /* Y increment. */
    double          zmin;	       /* Min grid value */
    double          zmax;	       /* Max grid value */
    double          dz;		       /* Contour increment. */
    char            order[8];	       /* grid order ("x" or "y") */
    char            mode[8];	       /* grid rendering mode */
    int             stride;	       /* grid stride. */
    float          *grid;	       /* grid values. */
}               GPLGrid;

typedef struct gpl_arrival {	       /* GPLArrival structure used in
				        * get_arrivals and put_arrival
				        * callbacks for passing arrival info
				        * to/from GatherPlot object. */
    void           *arrival_id;	       /* User defined arrival ID. */
    double          time;	       /* Arrival epoch time. */
    double          deltime;	       /* Arrival time uncertainty. */
    double          ptime;	       /* Arrival peak amplitude epoch time. */
    double          ap0time;	       /* Amp-per t0. */
    double          ap1time;	       /* Amp-per t1. */
    float           pamp;	       /* Arrival peak amplitude. */
    float           ap0amp;	       /* Amp-per amp0. */
    float           ap1amp;	       /* Amp-per amp1. */
    float           amp;	       /* Amp-per amp. */
    char            phase[16];	       /* Arrival phase code. */
    char            fm[8];	       /* Arrival first motion. */
    char            quality[8];	       /* Arrival quality. */
    int             overlay;	       /* Overlay arrival flag. */
    int             defining;	       /* Event defining arrival flag. */
    int             associated;	       /* Event associated arrival flag. */
    int             magnitude;	       /* Event magnitude arrival flag. */
    int		    done;
}               GPLArrival;

typedef struct gpl_pane_ {	       /* GPLPane structure contains info for
				        * a single window pane. */
    struct gather_plot_ *parent;       /* GatherPlot parent structure. */
    int             type;	       /* A type flag for identifying pane. */
    int             visible;	       /* Visibility flag. */
    int             grab;	       /* X-event grab flag. */
    int             x;		       /* Pane dimensions relative to win. */
    int             y;
    int             w;
    int             h;
    XPixel             background;	       /* Pane background color. */
    XPixel             foreground;	       /* Pane foreground color. */
}               GPLPane;

typedef struct gather_plot_ {	       /* GatherPlot structure contains state
				        * info and callback procedure
				        * pointers for a single GatherPlot
				        * object. */
    Display        *display;	       /* X11 display. */
    int             screen;	       /* X11 screen. */
    Visual         *visual;	       /* X11 visual. */
    int             depth;	       /* Depth of display. */
    Colormap        cmap;	       /* X11 colormap. */
    char            progname[64];      /* program name. */
    char            title[128];	       /* Window title. */
    int             shutdown;	       /* Shutdown flag. */
    int             first;	       /* First time to be drawn flag. */
    TkSend         *tks;	       /* PIC tks pointer */
    int             replyrequest;      /* Request for a reply flag. */
    int             fdcom;	       /* fd-based com out */
    struct gather_plot_static_ *Static;
    /* Static X-window info. */
    Ol             *ol;		       /* OpenLook widget set pointer. */
    Box            *box;	       /* Reverse video box object. */
    void           *overlays;	       /* Overlays object for drawing arrival
				        * flags. */
    struct gather_plot_ *gpl_mag;      /* A magnification window. */
    int             mag_win_w;	       /* Nominal magnification win. width. */
    int             mag_win_h;	       /* Nominal magnification win. height. */
    int             n_sln_wins;	       /* Number of active sln windows. */
    struct gather_plot_ **gpl_sln;     /* Selection windows. */
    int             sln_win_w;	       /* Nominal selection win. width. */
    int             sln_win_h;	       /* Nominal selection win. height. */
    struct gather_plot_ *gpl_parent;   /* A mag/sln window's parent. */
    char           *user_data;	       /* User data. */
    int             nwidgets;	       /* Number of widgets. */
    struct gpl_widgets_ {	       /* List of associated widgets. */
	void           *ol_widget;
	int             type;
	int             on;
    }               widgets[20];
    OlMenu         *trace_menu;	       /* Trace delete/select pull-down. */
    OlMenu         *phase_menu;	       /* Pop-up menu of phases. */
    OlMenu         *fm_menu;	       /* Pop-up menu of first motions. */
    OlMenu         *arrival_menu;      /* Pop-up menu of arrival editing. */
    OlScrollbar    *uvar_scrollbar;    /* uvar axis scrollbar. */
    OlScrollbar    *sta_scrollbar;     /* station scrollbar. */
    OlScrollbar    *time_scrollbar;    /* time axis scrollbar. */
    int             button_flags;      /* Botton on/off flags. */
    XPixel             label_foreground;  /* Foreground color for labeling. */
    XPixel             label_background;  /* background color for labeling. */
    XPixel             uvar_foreground;   /* Foreground color for uvar ann. */
    XPixel             trace_foreground;  /* Foreground color for trace window
				        * (and pixmap). */
    XPixel             trace_background;  /* Background color for trace window
				        * (and pixmap). */
    XPixel             timemark_foreground; /* Foreground color for timemark line */
    int             gpl_win_visible;   /* GPL window visibility flag. */
    int             batch_on;	       /* If set then defers drawing and/or
				        * painting until it is unset. */
    Window          gpl_win;	       /* X11 window for GPL object. */
    Window          gpl_win_parent;    /* Parent of gpl_win. */
    GC              gpl_win_gc;	       /* X11 graphics context used for
				        * drawing into gpl window. */
    GC              time_cursor_gc;    /* X11 graphics context used for
				        * drawing the time cursor. */
    GC              time_line_gc;      /* X11 graphics context used for
				        * drawing time lines into trace pane. */
    GC              timemark_line_gc;  /* X11 graphics context used for
				        * drawing time marks into trace pane. */
    GC              trace_bkg_gc;      /* X11 graphics context used for
				        * drawing trace pane background color */
    GC              trace_frg_gc;      /* X11 graphics context used for
				        * drawing trace pane foreground color */
    GC              trace_rect_gc;     /* X11 graphics context used for
				        * drawing trace rectangle color */
    GC              label_bkg_gc;      /* X11 graphics context used for
				        * drawing label background color. */
    GC              trace_moveout_gc;  /* X11 graphics context used for
				        * drawing moveout overlay. */
    GC              trace_arrival_gc;  /* X11 graphics context used for
				        * drawing arrival overlay. */
    GPLPane         parent_pane;       /* Parent window pane used to house
				        * parent window width and height for
				        * resizing. */
    GPLPane         gpl_pane;	       /* Total gpl window pane. */
    float           gpl_win_x_fr;      /* Total gpl window x, y, width and
				        * height as fractions of the parent
				        * window. */
    float           gpl_win_y_fr;
    float           gpl_win_w_fr;
    float           gpl_win_h_fr;
    GPLPane         trace_pane;	       /* Trace display pane. */
    float           trace_pane_x_fr;   /* Trace display pane x, y, width and
				        * height as fractions of the total
				        * gpl window. */
    float           trace_pane_y_fr;
    float           trace_pane_w_fr;
    float           trace_pane_h_fr;
    int             trace_pane_l_margin;	/* Minimum left, right, top
						 * and bottom margins in
						 * pixels between the total
						 * gpl window edges and the
						 * trace display pane. */
    int             trace_pane_r_margin;
    int             trace_pane_t_margin;
    int             trace_pane_b_margin;
    GPLPane         widget_pane;       /* Button and pull-down menu pane. */
    GPLPane         uvar_scrl_pane;    /* Uvar axis scrollbar pane. */
    GPLPane         uvar_pane;	       /* Uvar axis annotation pane. */
    GPLPane         sta_pane;	       /* Station annotation pane. */
    GPLPane         sta_scrl_pane;     /* Station scrollbar pane. */
    GPLPane         time_pane;	       /* Time axis annotation pane. */
    GPLPane         status_pane;       /* Status/typein pane. */
    GPLPane         time_scrl_pane;    /* Time axis scrollbar pane. */
    int             trace_win_imm;     /* Immediate display flag. */
    int             trace_mouse_mode;  /* Determines how GPL_event will
				        * process mouse events within the
				        * trace display pane. =
				        * GPL_TIME_SCROLL - Mouse buttons
				        * will cause time scrolling. */
    int             trace_mouse_mode_old;	/* Used as an internal state
						 * variable in conjunction
						 * with GPL_MOUSE_RESTORE. */
    int             trace_mouse_state; /* Used as an internal state variable
				        * for controlling mouse events. */
    int             trace_mouse_xold;  /* Mouse x position when Button2 is
				        * pressed. */
    int             trace_mouse_yold;  /* Mouse y position when Button2 is
				        * pressed. */
    int             trace_x_first;     /* First button click place holder. */
    int             trace_y_first;     /* First button click place holder. */
    int             arrival_select;    /* Arrival selection flag. */
    int             arrival_edit;      /* Arrival editing flag. */
    unsigned long int old_mask;	       /* Previous event mask used in zooming */
    int		    n_time_marks;      /* number of timing marks */
    double	    time_marks[100];   /* times for timing marks */
    void           *time_cursor_ovls;  /* Time cursor overlays. */
    unsigned int    time_flags;	       /* Annotation flags bitwised or'ed
				        * GPL_ON_LABEL	- If set time
				        * annotation visible. GPL_ON_GRID
				        * - If set time grid lines visible.
				        * GPL_ATTACHED	- If set time
				        * annotation attached to trace
				        * sub-window. */
    int             ntimes;	       /* Number of timing annotation lines. */
    int             xtime;	       /* x-value of first time line. */
    int             dxtime;	       /* delta x value of time lines. */
    double          ttime;	       /* Time of first time line. */
    double          dttime;	       /* Time line time increment. */
    double          tstime;
    unsigned int    uvar_flags;	       /* Annotation flags bitwised or'ed
				        * GPL_ON_LABEL	- If set uvar
				        * annotation visible. GPL_ON_GRID
				        * - If set uvar grid lines visible.
				        * GPL_ORDINAL	- If set uvar axis
				        * numbering is ordinal GPL_RIGHT
				        * - If set uvar pane is on the right
				        * of the trace sub-window.
				        * GPL_ATTACHED	- If set uvar
				        * annotation attached to trace
				        * sub-window. */
    int             uvar_ordinal;      /* If true, then uvar axis is
				        * ordinally numbered (automatic). */
    int             nuvars;	       /* Number of uvar annotation lines. */
    int             yuvar;	       /* y-value of first uvar line. */
    int             dyuvar;	       /* delta y value of uvar lines. */
    float           uuvar;	       /* Uvar of first uvar line. */
    float           duuvar;	       /* Uvar line uvar increment. */
    float           usuvar;
    unsigned int    sta_flags;	       /* Annotation flags bitwised or'ed
				        * GPL_ON_LABEL	- If set sta
				        * annotation visible. GPL_ON_GRID
				        * - If set sta grid lines visible.
				        * GPL_LEFT	- If set sta pane is
				        * on the left of the trace
				        * sub-window. GPL_ATTACHED	- If
				        * set sta annotation attached to
				        * trace sub-window. */
    Pixmap          trace_pixmap;      /* X11 pixmap for storing the
				        * trace_win trace image. Used as
				        * backing for trace_win to avoid
				        * redrawing the actual traces. */
    GC              trace_pixmap_gc;   /* X11 graphics context used for
				        * drawing into trace_pixmap. */
    GC              trace_pixmap_gc2;  /* X11 graphics context used for
				        * drawing into trace_pixmap. */
    int             trace_pixmap_w;    /* Width of trace_pixmap. Should match
				        * trace_win_w or a new pixmap is
				        * needed. */
    int             trace_pixmap_h;    /* Height of trace_pixmap. Should
				        * match trace_win_h or a new pixmap
				        * is needed. */
    int             trace_plot_mode;   /* Plotting mode for the trace window.
				        * = GPL_WIGGLE_PLOT - Normal "wiggle"
				        * plots. = GPL_GRAYSCALE_PLOT - Gray
				        * shaded amplitudes with red
				        * indicating clipping. */
    int             trace_overlay_mode;/* Plotting mode for overlay plots. =
				        * GPL_OVERLAY_CONTINUOUS - Continuous
				        * line plot. = GPL_OVERLAY_DISCRETE -
				        * Discrete time lines across each
				        * trace pane. */
    int             trace_moveout_mode;/* Plotting mode for the time moveout.
				        * = GPL_NONE - No moveout correction.
				        * = GPL_LINEAR - Linear correction
				        * with uvar. = GPL_HYPERBOLIC -
				        * Hyperbolic correction with uvar. */
    double          trace_moveout_t0;  /* Time corresponding to uvar = 0. */
    float           trace_moveout_slo; /* Moveout slowness. */
    double          trace_moveout_t0l; /* Time corresponding to uvar = 0. */
    float           trace_moveout_slol;/* Moveout slowness. */
    int             trace_filter_mode; /* Trace filtering mode. = GPL_OFF -
				        * No filtering. = GPL_ON - Filter
				        * according to trace_filter_index. */
    int             trace_filter_index;/* Index into filters array that
				        * defines the current filter. */
    int             trace_time_resize; /* Time axis resize flag. Determines
				        * how resizing effects time plot
				        * scaling. = GPL_FIT	- Fit twin to
				        * new size. = GPL_FIXED	- Maintain
				        * prev scale. */
    int             trace_uvar_resize; /* Uvar axis resize flag. Determines
				        * how resizing effects uvar plot
				        * scaling. = GPL_FIT	- Fit uvarwin
				        * to new size. = GPL_FIXED	-
				        * Maintain prev scale. */
    double          t0;		       /* Reduced epoch time corresponding to
				        * the left hand edge of the trace
				        * window plot sub-region. */
    double          twin;	       /* Time window corresponding to the
				        * width of the trace window plot
				        * sub-region. */
    double          dist;	       /* Distance for computing velocities */
    double          torig;	       /* Origin time for computing
				        * velocities */
    double          t0_full;
    double          twin_full;
    char           *uvar_name;	       /* Character string name that
				        * identifies "uvar". */
    float           uvar0;	       /* uvar value corresponding to the top
				        * edge of the trace window plot
				        * sub-region. */
    float           uvarwin;	       /* uvar window corresponding to the
				        * height of the trace window plot
				        * sub-region. */
    float           uvar0_full;
    float           uvarwin_full;
    int             sta0;	       /* Station index at top of plot rgn. */
    int             stawin;	       /* No. of stations in plot rgn. */
    int             stawin_full;       /* Maximum no. of stations. */
    int             stamax;            /* Maximum no. of stations to display. */
    float           trace_ht;	       /* Single trace height as a fraction
				        * of one "normalized" trace width. */
    float           trace_height;      /* Single trace height as a fraction
				        * of the trace window plot sub-region
				        * height. */
    float           amp0;	       /* Reduced amplitude value that maps
				        * to the value of uvar for each
				        * trace. */
    float           ampwin;	       /* Reduced amplitude window that
				        * corresponds to trace_height for
				        * each trace. */
    float           gain;	       /* Amplitude gain adjustment. */
    int             clip;	       /* Amplitude clipping flag. */
    int             amp_scale_mode;    /* Amplitude scaling mode. = GPL_FIXED
				        * - Fixed scale according to ampwin.
				        * = GPL_AUTO	- Auto-scale each
				        * trace independently. = GPL_AUTO1
				        * - Auto-scale according to first
				        * trace. = GPL_AUTOA	- Auto-scale
				        * according to all traces. = GPL_AGC
				        * - Time varying automatic gain
				        * control. */
    float           agcwin;	       /* Automatic gain control time window
				        * for RMS averaging. */
    int             amp_uvar_mode;     /* Amp-uvar scaling mode. = GPL_OFF
				        * - No amp-uvar scaling. =
				        * GPL_AMP_SQUAREROOT	- gain =
				        * 1/sqrt(uvar) = GPL_AMP_LINEAR	-
				        * gain = 1/uvar = GPL_AMP_SQUARED
				        * - gain = 1/uvar**2 */
    int             ntraces;	       /* Total number of traces. */
    struct strace_ {		       /* Array of strace structures each
				        * defining a single trace. */
	void           *trace_id;      /* A pointer to user defined trace id. */
	float           ampwin;	       /* Amplitude window. */
	float           amp0;	       /* Amplitude 0 value. */
	float           ascale;	       /* Amplitude plot scale. */
	double          t0;	       /* Start time. */
	int             xl;	       /* Station label box dims. */
	int             xr;
	int             yt;
	int             yb;
	int             select;	       /* Selection flag and order. */
	int             display;       /* Display flag and order. */
	int	        filter_index;  /* filter index */
	int	        filter_mode;   /* filter mode */
    }              *traces;
    double          tshift;	       /* Internal time shift used to
				        * implement quick time scrolling. */
    struct gpl_arrival_flag_ *arrfl_sel; /* Current selected arrival. */
    int deleted;
    int             (*get_trace) ();   /* Single trace callback procedure
				        * used to get trace samples for a
				        * single trace. The procedure is
				        * called as follows:
				        * 
				        * int get_trace (gpl,trace_id,t0,twin,
				        * npixels,gpl_trace) GatherPlot *gpl;
				        * char *         trace_id; double
				        * t0; double                     twin
				        * int       npixels; GPLTrace *
				        * gpl_trace;
				        * 
				        * where:
				        * 
				        * gpl		= (i) GatherPlot obj.
				        * trace_id	= (i) User def trace
				        * id. t0		= (i) Plot
				        * abs start tm twin		= (i)
				        * Plot time window. npixels	= (i)
				        * No. time pixels.
				        * gpl_trace->id_string	= (o) Station
				        * id. gpl_trace->units	= (o) Trace
				        * amp. units. gpl_trace->uvar
				        *  (o) Trace uvar value.
				        * gpl_trace->nsegs	= (o) No.
				        * trace segs. gpl_trace->tstart	= (o)
				        * Seg. start times. gpl_trace->dt
				        *  (o) Seg. samp incrs.
				        * gpl_trace->nsamps	= (o) Seg. #
				        * of samples. gpl_trace->data
				        *  (o) Seg. trace samples */
    int             (*get_grid) ();    /* Single trace callback procedure
				        * used to get a background grid to
				        * show as a color contour. The
				        * procedure is called as follows:
				        * 
				        * int get_grid (gpl,trace_id,t0,twin,
				        * npixels,gpl_grid) GatherPlot *gpl;
				        * char *         trace_id; double
				        * t0; double                     twin
				        * int       npixels; GPLGrid *
				        * gpl_grid;
				        * 
				        * where:
				        * 
				        * gpl		= (i) GatherPlot obj.
				        * trace_id	= (i) User def trace
				        * id. t0		= (i) Plot
				        * abs start tm twin		= (i)
				        * Plot time window. npixels	= (i)
				        * No. time pixels. */
    int             (*get_arrivals) ();/* Single trace callback procedure
				        * used to get arrival times and
				        * labels for a single trace. The
				        * procedure is called as follows:
				        * 
				        * int get_arrivals (trace_id, t0, twin,
				        * npixels, narrivals, arrival_ids,
				        * times, phases, fms, ptimes, pamps,
				        * ap0times, ap0amps, ap1times,
				        * ap1amps) char *        trace_id;
				        * double                  t0; double
				        * twin; int           npixels; int *
				        * narrivals; char ***
				        * arrival_ids; double **
				        * times; char ***      phases; char ***
				        * fms; double **     ptimes; float **
				        * pamps; double **     ap0times;
				        * float **                ap0amps;
				        * double **     ap1times; float **
				        * ap1amps;
				        * 
				        * where:
				        * 
				        * trace_id     = (i) User def trace id.
				        * t0           = (i) Plot abs start
				        * tm twin         = (i) Plot time
				        * window. npixels      = (i) No. time
				        * pixels. narrivals	= (o) No.
				        * arrivals. arrival_ids	= (o) Arrival
				        * IDs. times	= (o) Arrival times.
				        * phases	= (o) Arrival phases.
				        * fms		= (o) Arrival fst
				        * mtns. ptimes	= (o) Arrival peak
				        * times pamps	= (o) Arrival peak
				        * amps ap0times	= (o) Amp-per t0.
				        * ap0amps	= (o) Amp-per amp0.
				        * ap1times	= (o) Amp-per t1.
				        * ap1amps	= (o) Amp-per amp1. */
    int             (*get_arrs) ();    /* Single trace callback procedure
				        * used to get arrival times and
				        * labels for a single trace. The
				        * procedure is called as follows:
				        * 
				        * int get_arrs (trace_id, t0, twin,
				        * npixels, narrivals, gpl_arrivals)
				        * char *        trace_id; double
				        * t0; double
				        * twin; int           npixels; int *
				        * narrivals; GPLArrival
				        * **gpl_arrivals;
				        * 
				        * where:
				        * 
				        * trace_id     = (i) User def trace id.
				        * t0           = (i) Plot abs start
				        * tm twin         = (i) Plot time
				        * window. npixels      = (i) No. time
				        * pixels. narrivals	= (o) No.
				        * arrivals. gpl_arrivals[].arrival_id
				        * = (o) User def arrival ID.
				        * gpl_arrivals[].time = (o) Arrival
				        * times. gpl_arrivals[].deltime = (o)
				        * Arrival time uncs. etc. */
    char           *(*put_arrival) (); /* Single trace callback procedure
				        * used to give an arrival time back
				        * to the user after it has been
				        * modified. The procedure is called
				        * as follows:
				        * 
				        * char * put_arrival  (arrival_id,
				        * trace_id, delete_flg, add_flg,
				        * time, phase, fm, ptime, pamp,
				        * ap0time, ap0amp, ap1time, ap1amp)
				        * char *        arrival_id; char *
				        * trace_id; int	         delete_flg;
				        * int 	                     add_flg;
				        * double        time; char *
				        * phase; char *
				        * fm; double        ptime; float
				        * pamp; double        ap0time; float
				        * ap0amp; double        ap1time;
				        * float                   ap1amp;
				        * 
				        * where:
				        * 
				        * arrival_id	= (i) Arrival ID.
				        * trace_id     = (i) Trace id.
				        * delete_flg	= (i) Delete arr.
				        * flag. add_flg	= (i) Add arr. flag.
				        * time		= (i) Arrival time.
				        * phase	= (i) Arrival phase. fm
				        *  (i) Arrival fst mtn. ptime	= (i)
				        * Arrival peak time pamp
				        *  (i) Arrival peak amp ap0time	= (i)
				        * Amp-per t0. ap0amp	= (i) Amp-per
				        * amp0. ap1time	= (i) Amp-per t1.
				        * ap1amp	= (i) Amp-per amp1.
				        * 
				        * put_arrival() should return the
				        * possibly new arrival_id or NULL if
				        * there is an error. */
    char           *(*put_arr) ();     /* Single trace callback procedure
				        * used to give an arrival time back
				        * to the user after it has been
				        * modified. The procedure is called
				        * as follows:
				        * 
				        * char * put_arr  (gpl, arrival_id,
				        * trace_id, delete_flg, add_flg,
				        * gpl_arrival) GatherPlot *  gpl;
				        * char *        arrival_id; char *
				        * trace_id; int	         delete_flg;
				        * int 	                     add_flg;
				        * GPL_Arrival * gpl_arrival;
				        * 
				        * where:
				        * 
				        * arrival_id	= (i) Arrival ID.
				        * trace_id     = (i) Trace id.
				        * delete_flg	= (i) Delete arr.
				        * flag. add_flg	= (i) Add arr. flag.
				        * gpl_arrival->time time
				        *  (i) Arrival time. etc.
				        * 
				        * put_arrival() should return the
				        * possibly new arrival_id or NULL if
				        * there is an error. */
    int             (*resize_proc) (); /* User defined callback procedure
				        * used whenever trace window total
				        * size has changed. The procedure is
				        * called as follows:
				        * 
				        * int resize_proc (gpl, w, h) GatherPlot
				        * *gpl; int               w; int
				        * h;
				        * 
				        * where:
				        * 
				        * gpl	= (i) Associated GatherPlot
				        * obj w	= (i) Window new total width.
				        * h	= (i) Window new total
				        * height. */
    int             (*key_event_proc) ();	/* User defined callback
						 * procedure used with
						 * keyboard events. The
						 * procedure is called as
						 * follows:
						 * 
						 * int key_event_proc (gpl,
						 * event, key, nb) GatherPlot *
						 * gpl; XEvent *
						 * event; char *
						 * key; int
						 * nb;
						 * 
						 * where:
						 * 
						 * gpl	= (i) Associated
						 * GatherPlot obj event= (i)
						 * X event. key	= (i) Key
						 * buffer. nb	= (i) Key
						 * buffer size.
						 * 
						 * key_event_proc() should
						 * return 1 if the key event
						 * is used or 0 otherwise. */
    int             (*sta_info) ();    /* User defined callback procedure
				        * used to list station-channel
				        * characteristics. The procedure is
				        * called as follows:
				        * 
				        * int sta_info (trace_id, nlines, lines)
				        * char *    trace_id; int *
				        * nlines; char ***
				        * lines;
				        * 
				        * where:
				        * 
				        * trace_id     = (i) User def trace id.
				        * nlines	= (o) Number of
				        * lines. lines	= (o) lines. */
}               GatherPlot;

struct gather_plot_static_ {	       /* This structure contains "static"
				        * X-windows stuff that only needs to
				        * be allocated the first time a
				        * GatherPlot object is created. Note
				        * that for now we do not allow
				        * display, visual, depth, cmap, etc.
				        * to change between calls to
				        * GPL_create. */
    Display        *display;	       /* X11 display. */
    int             screen;	       /* X11 screen. */
    Visual         *visual;	       /* X11 visual. */
    int             depth;	       /* Depth of display. */
    Colormap        cmap;	       /* X11 colormap. */
    XPixel   pixel;	       /* If this value is non-zero, then
				        * this is a read/write pixel that has
				        * been allocated along with its
				        * complement. */
    XPixel   pixel_cmpl;	       /* Complement to pixel. */
    XPixel   white_pixel;       /* White pixel. */
    XPixel   black_pixel;       /* Black pixel. */
    XPixel   pink_pixel;	       /* Pink pixel. */
    XPixel   red_pixel;	       /* Red pixel. */
    XPixel   yellow_pixel;      /* Yellow pixel. */
    XPixel   yellow2_pixel;     /* Yellow pixel. */
    XPixel   green_pixel;       /* Green pixel. */
    XPixel   blue_pixel;	       /* Blue pixel. */
    XPixel   gray_pixel;	       /* 50% gray pixel. */
    XPixel   gray_pixels[32];   /* Pixels used for gray scaling. */
    XPixel   chrom_pixels[32];  /* Pixels used for color scaling. */
    XPixel   label_foreground;  /* Foreground color for labeling. */
    XPixel   label_background;  /* background color for labeling. */
    XPixel   uvar_foreground;   /* Foreground color for uvar ann. */
    XPixel   trace_foreground;  /* Foreground color for trace window
				        * (and pixmap). */
    XPixel   trace_background;  /* Background color for trace window
				        * (and pixmap). */
    XPixel   arrival_foreground;/* Foreground color for arrival flags */
    XPixel   arrival_background;/* Background color for arrival flags */
    XPixel   def_arrival_foreground;/* Foreground color for event defining arrival flags */
    XPixel   def_arrival_background;/* Background color for event defining arrival flags */
    XPixel   ass_arrival_foreground;/* Foreground color for event associated arrival flags */
    XPixel   ass_arrival_background;/* Background color for event associated arrival flags */
    XPixel   mag_arrival_foreground;/* Foreground color for event magnitude arrival flags */
    XPixel   mag_arrival_background;/* Background color for event magnitude arrival flags */
    XPixel   sel_arrival_foreground;	/* Foreground color for
						 * selected arrival flags */
    XPixel   sel_arrival_background;	/* Background color for
						 * selected arrival flags */
    XPixel   pred_arrival_foreground;	/* Foreground color for
						 * predicted arrival flags */
    XPixel   detection_foreground;	/* Foreground color for
						 * detection flags */
    XPixel   timemark_foreground;	/* Foreground color for
						 * timemark line */
    Pixmap          dotted_pixmap;     /* Pixmap for drawing dotted lines. */
    XFontStruct    *trace_font;	       /* Font used in the trace window. */
    XFontStruct    *trace_font_small;  /* Small font used in the trace win. */
    Cursor          normal_cursor;     /* Normal trace window cursor. */
    Cursor          busy_cursor;       /* Window busy cursor. */
    Cursor          hscrl_cursor;      /* Horizontal scrolling cursor. */
    Cursor          vscrl_cursor;      /* Vertical scrolling cursor. */
    Tbl            *filters;
};

struct gather_plot_filter_ {	       /* This structure contains the
				        * GatherPlot "internal" filter info
				        * for a single filter. */
    int             type;	       /* Filter type. = GPL_BUTTERWORTH -
				        * Bandpass, lowpass or highpass
				        * Butterworth filter with: param1 =
				        * Low cutoff freq. param2 = Low
				        * cutoff order. param3 = High cutoff
				        * freq. param4 = High cutoff order.
				        * Low pass if param1 = 0.0. High pass
				        * if param3 = 0.0. = GPL_WAA -
				        * Wood-Anderson acceleration response
				        * filter. = GPL_WAV - Wood-Anderson
				        * velocity response filter. = GPL_WAD
				        * - Wood-Anderson displacement
				        * response filter. */
    char            label[32];	       /* A filter label used with the menu
				        * buttons and for other annotations. */
    double          start_pad;	       /* Defines filter transient time at
				        * the start of the trace. */
    double          end_pad;	       /* Defines filter transient time at
				        * the end of the trace. */
    float           param1;	       /* Filter parameters. */
    float           param2;
    float           param3;
    float           param4;
    char            filter_string[512];
};

typedef struct gpl_arrival_flag_ {
    int		    type;
    void           *arrival_id;
    struct strace_ *strace;
    GatherPlot     *gpl;
    int             selected;
    int             overlay;
    Pixmap          pm;
    int             h;
    double          time;
    double          deltime;
    Box            *deltime_box;
    double          ptime;
    double          ap0time;
    double          ap1time;
    float           pamp;
    float           amp;
    float           ap0amp;
    float           ap1amp;
    Box            *ampper_box;
    char            phase[16];
    char            fm[8];
    int		    defining;
    int		    associated;
    int		    magnitude;
}               gpl_arrival_flag;

typedef struct gpl_timemark_flag_ {
    int		    type;
    GatherPlot     *gpl;
    int	           tmark_index;
}		gpl_timemark_flag;


/*
 *	Following are GatherPlot attributes (uset in GPL_set).
 */

/* X11 window definitions. */
#define	GPL_WINDOW		1      /* GPL window id. */
#define	GPL_PARENT_WINDOW	2      /* GPL parent window. */
#define	GPL_TITLE		92     /* GPL window title. */
/* Callback procedures. */
#define	GPL_TRACE_PROC		3      /* Callback to get trace samples. */
#define	GPL_GRID_PROC		94     /* Callback to get grid data. */
#define	GPL_RESIZE_PROC		4      /* Callback when trace window resizes. */
#define	GPL_ARRIVAL_PROC	5      /* Callback to get arrival times. */
#define	GPL_ARR_PROC		84     /* Callback to get arrival times. */
#define	GPL_PUT_ARRIVAL_PROC	6      /* Callback to put arrival times. */
#define	GPL_PUT_ARR_PROC	85     /* Callback to put arrival times. */
#define	GPL_KEY_EVENT_PROC	7      /* Keyboard event callback procedure. */
#define	GPL_STA_INFO_PROC	8      /* Station info callback procedure. */
/* Trace definitions. */
#define	GPL_TRACES		9      /* List of all traces (user ptrs). */
#define	GPL_TRACE		10     /* Single trace user ptr. */
/* User data definitions. */
#define	GPL_USER_DATA		11     /* User defined data. */
/* General window layout attributes. */
#define	GPL_GPL_WIN_X_FR	12     /* Total gpl window x location as a
				        * fraction of the parent window width */
#define	GPL_GPL_WIN_Y_FR	13     /* Total gpl window y location as a
				        * fraction of the parent window ht. */
#define	GPL_GPL_WIN_W_FR	14     /* Total gpl window width as a
				        * fraction of the parent window width */
#define	GPL_GPL_WIN_H_FR	15     /* Total gpl window height as a
				        * fraction of the parent window ht. */
#define	GPL_TRACE_PANE_X_FR	16     /* Trace display pane x location as a
				        * fraction of the gpl window width. */
#define	GPL_TRACE_PANE_Y_FR	17     /* Trace display pane y location as a
				        * fraction of the gpl window ht. */
#define	GPL_TRACE_PANE_W_FR	18     /* Trace display pane width as a
				        * fraction of the gpl window width. */
#define	GPL_TRACE_PANE_H_FR	19     /* Trace display pane height as a
				        * fraction of the gpl window ht. */
#define	GPL_TRACE_PANE_L_MARGIN	20     /* Trace display pane left margin in
				        * pixels relative to gpl window. */
#define	GPL_TRACE_PANE_R_MARGIN	21     /* Trace display pane right margin in
				        * pixels relative to gpl window. */
#define	GPL_TRACE_PANE_T_MARGIN	22     /* Trace display pane top margin in
				        * pixels relative to gpl window. */
#define	GPL_TRACE_PANE_B_MARGIN	23     /* Trace display pane bottom margin in
				        * pixels relative to gpl window. */
/* Generic pane attributes. */
#define	GPL_PANE_VISIBLE	24     /* Pane visibility. */
#define	GPL_PANE_FOREGROUND	25     /* Pane foreground color. */
#define	GPL_PANE_BACKGROUND	26     /* Pane background color. */
/* Widget pane attributes. */
#define	GPL_BUTTON_ON		27     /* Individual menu button on/off. */
#define	GPL_BUTTONS_ON		28     /* libol buttons on/off. */
#define	GPL_ADD_WIDGET		29     /* Add user defined ol widget. */
/* Uvar pane attributes. */
#define	GPL_UVAR_NAME		30     /* "User Variable" name. */
#define	GPL_UVAR_LABEL_FLAGS	31     /* Uvar axis labeling flags. */
#define	GPL_UVAR_SCROLL_INCR	32     /* Uvar axis scroll increment. */
/* Sta pane attributes. */
#define	GPL_STA_LABEL_FLAGS	33     /* Station-channel labeling flags. */
/* Trace pane attributes. */
#define	GPL_TRACE_PLOT_MODE	34     /* Trace plotting mode. */
#define	GPL_T0			35     /* Reduced time at left of plot regn. */
#define	GPL_TWIN		36     /* Reduced time window. */
#define	GPL_T0_FULL		37     /* "Full" time window t0. */
#define	GPL_TWIN_FULL		38     /* "Full" time window twin. */
#define	GPL_UVAR0		39     /* User var. at top of plot region. */
#define	GPL_UVARWIN		40     /* User variable window. */
#define	GPL_UVAR0_FULL		41     /* "Full" uvar window uvar0. */
#define	GPL_UVARWIN_FULL	42     /* "Full" uvar window uvarwin. */
#define	GPL_T_BOTH		43     /* Time axis scaling. */
#define	GPL_UVAR_BOTH		44     /* Uvar axis scaling. */
#define	GPL_UVAR_ORDINAL	45     /* uvar axis ordinal trace no. */
#define	GPL_AMP0		46     /* Reduced trace amplitude value that
				        * maps to uvar. */
#define	GPL_AMPWIN		47     /* Reduced trace amplitude window that
				        * maps to a single trace height. */
#define	GPL_TRACE_HEIGHT	48     /* Single trace height in fractions of
				        * the trace plot sub-region height. */
#define	GPL_TRACE_TIME_RESIZE	49     /* Time axis resize flag. */
#define	GPL_TRACE_UVAR_RESIZE	50     /* Uvar axis resize flag. */
#define	GPL_AMP_SCALE_MODE	51     /* Amplitude scaling mode. */
#define	GPL_TRACE_MOUSE_MODE	52     /* Mouse behavior mode in trace window */
#define	GPL_TRACE_MOVEOUT_MODE	53     /* Moveout plotting mode. */
#define	GPL_TRACE_FILTER_MODE	54     /* Trace filtering mode. */
#define	GPL_TRACE_FILTER_INDEX	55     /* Trace filter index. */
#define	GPL_TRACE_CLIPPING_FLAG	56     /* Trace amp clipping flag. */
#define	GPL_TRACE_GAIN		57     /* Trace amp gain. */
#define	GPL_AMP_UVAR_MODE	58     /* Amp-uvar scaling mode. */
#define	GPL_TRACE_MAGNIFY	59     /* Magnify an individual trace. */
#define	GPL_TRACE_DATA		60     /* Get trace data (GPL_get only) */
/* Time pane attributes. */
#define	GPL_TIME_LABEL_FLAGS	61     /* Time axis labeling flags. */
#define	GPL_T_SCROLL_INCR	62     /* Time axis scroll increment. */
/* Arrival flag attributes. */
#define	GPL_ARRIVAL_BACK_COLOR	63     /* Arrival flag background color. */
#define	GPL_ARRIVAL_FORE_COLOR	64     /* Arrival flag foreground color. */
#define	GPL_ARRIVAL_SELECT	65     /* Enable/disable arrival flag
				        * selection with mouse. */
#define	GPL_ARRIVAL_EDIT	66     /* Enable/disable arrival flag editing
				        * with mouse. */
#define	GPL_SEL_ARRIVAL_BACK_COLOR	67
 /* Selected arrival flag back color. */
#define	GPL_SEL_ARRIVAL_FORE_COLOR	68
 /* Selected arrival flag fore color. */
#define	GPL_PRED_ARRIVAL_FORE_COLOR	95
 /* Predicted arrival flag fore color. */
#define	GPL_DETECTION_FORE_COLOR	96
 /* detection flag fore color. */
#define	GPL_TIMEMARK_FORE_COLOR		105
 /* detection flag fore color. */
#define	GPL_ARRIVAL_TIME	69     /* Arrival time. */
#define	GPL_ARRIVAL_DELTIM	83     /* Arrival deltime. */
#define	GPL_ARRIVAL_DELTIM_SHOW	86     /* Arrival show deltime. */
#define	GPL_ARRIVAL_AMPPER	90     /* Arrival amp-per. */
#define	GPL_ARRIVAL_AMPPER_AUTO	82     /* Arrival show amp-per. */
#define	GPL_ARRIVAL_AMPPER_SHOW	91     /* Arrival show amp-per. */
#define	GPL_ARRIVAL_PHASE	70     /* Arrival phase. */
#define	GPL_ARRIVAL_FM		99     /* Arrival phase. */
#define	GPL_ARRIVAL_FMAUTO	100    /* Arrival phase. */
#define	GPL_ARRIVAL_DELETE	71     /* Delete arrival. */
#define	GPL_ARRIVAL_SELECTED	72     /* Mark arrival as selected. */
#define	GPL_ARRIVAL_ADD		73     /* Add an arrival. */
#define	GPL_TMARK_ADD		101    /* Add a time mark. */
#define	GPL_TMARK_TIME		102    /* Set time mark time. */
#define	GPL_TMARK_DELETE	103    /* Delete time mark. */
#define	GPL_TMARK_DELETE_ALL	104    /* Delete all time marks. */
#define	GPL_PHASE_NAMES		74     /* Phase menu names. */

#define	GPL_NEW_SLN_WINDOW	75     /* New selection window. */
#define	GPL_STA_LEFT		76     /* Put station annotations on left. */
#define	GPL_STA0		77     /* Station index at top of plot rgn. */
#define	GPL_STAWIN		78     /* No. of stations in  plot rgn. */
#define	GPL_STA_BOTH		79     /* station window scaling. */
#define	GPL_SLN_WINDOW		80     /* Selection window. */
#define	GPL_STA_POSITION	81     /* Current station pos. (GPL_get only) */
#define	GPL_TKS_CREATE		97     /* Create tcl/tk link */
#define	GPL_TKS_REGISTER	98     /* Create tcl/tk registry */
#define	GPL_COMOUT_FD		89     /* Set output com file descriptor */
#define	GPL_VELOCITY		93     /* Set distance and origin time */
#define	GPL_TRACE_NUP		106
#define	GPL_TRACE_UP		107
#define	GPL_DEFINING_ARRIVAL_BACK_COLOR	110     /* Event defining arrival flag background color. */
#define	GPL_DEFINING_ARRIVAL_FORE_COLOR	111     /* Event defining arrival flag foreground color. */
#define	GPL_ASSOCIATED_ARRIVAL_BACK_COLOR	112     /* Event associated arrival flag background color. */
#define	GPL_ASSOCIATED_ARRIVAL_FORE_COLOR	113     /* Event associated arrival flag foreground color. */
#define	GPL_MAGNITUDE_ARRIVAL_BACK_COLOR	114     /* Event magnitude arrival flag background color. */
#define	GPL_MAGNITUDE_ARRIVAL_FORE_COLOR	115     /* Event magnitude arrival flag foreground color. */
#define	GPL_TRACES_TBL		116      /* List of all traces (Tbl of ptrs). */
#define	GPL_BATCH		117      /* controls batch display setup */
#define	GPL_TRACES_NUP_TBL	118      /* List of all traces (Tbl of ptrs). */

#define	GPL_TMARK_NULL		-999999999.999

/*
 *	Following are GatherPlot pane types.
 */

#define	GPL_PARENT_PANE		0
#define	GPL_GPL_PANE		1
#define	GPL_TRACE_PANE		2
#define	GPL_WIDGET_PANE		3
#define	GPL_UVAR_SCRL_PANE	4
#define	GPL_UVAR_PANE		5
#define	GPL_STA_PANE		6
#define	GPL_STA_SCRL_PANE	7
#define	GPL_TIME_PANE		8
#define	GPL_STATUS_PANE		9
#define	GPL_TIME_SCRL_PANE	10

/*
 *	Following are overlay types.
 */

#define	GPL_OVERLAY_TMARK	0
#define	GPL_OVERLAY_ARRIVAL	1

/*
 *	Following are GatherPlot plot modes.
 */

#define	GPL_WIGGLE		0
#define	GPL_GRAYSCALE		1

/*
 *	Following are Amplitude scaling modes.
 */

#define	GPL_FIT			0
#define	GPL_FIXED		1
#define	GPL_AUTO		2
#define	GPL_AUTO0		3
#define	GPL_AUTO1		4
#define	GPL_AUTOA		5
#define	GPL_AGC			6

/*
 *	Following are mouse behavior modes.
 */

#define	GPL_DEFER		0      /* Defer mouse events for other
				        * processing. */
#define	GPL_TIME_SCROLL		1
#define	GPL_TIME_ZOOMIN		2
#define	GPL_TIME_ZOOMOUT	3
#define	GPL_UVAR_SCROLL		4
#define	GPL_UVAR_ZOOMIN		5
#define	GPL_UVAR_ZOOMOUT	6
#define	GPL_BOTH_SCROLL		7
#define	GPL_BOTH_ZOOMIN		8
#define	GPL_BOTH_ZOOMOUT	9
#define	GPL_MOVEOUT		10
#define	GPL_ADD_ARRIVALS	11
#define	GPL_ADD_TMARKS		12
#define	GPL_RESTORE		13
#define GPL_ARRIVAL_RANGE	14

/*
 *	Following are time moveout behavior modes.
 */

#define	GPL_NONE		0
#define	GPL_LINEAR		1
#define	GPL_HYPERBOLIC		2

/*
 *	Following are amp-uvar scaling modes.
 */

#define	GPL_LINEAR		1
#define	GPL_SQUAREROOT		2
#define	GPL_SQUARED		3

/*
 *	Following are filter types and modes.
 */

#define	GPL_BUTTERWORTH		1
#define	GPL_WAV			2
#define	GPL_WAD			3
#define	GPL_WAA			4
#define	GPL_FILSTRING		5

/*
 *	Following are mouse states.
 */

#define	GPL_MOUSE0		0
#define	GPL_MOUSE1		1
#define	GPL_MOUSE2		2
#define	GPL_MOUSE3		3
#define	GPL_MOUSE1B1		4
#define	GPL_MOUSE1B3		5

/*
 *	Following are widget indices into the gpl->widgets array.
 */

#define	GPL_NWIDGETS		8
#define	GPL_QUIT_WIDGET		0
#define	GPL_MOUSE_WIDGET	1
#define	GPL_AMP_WIDGET		2
#define	GPL_MODE_WIDGET		3
#define	GPL_MOVEOUT_WIDGET	4
#define	GPL_FILTER_WIDGET	5
#define	GPL_ADD_ARRIVAL_WIDGET	6
#define	GPL_ADD_TMARK_WIDGET	7
#define	GPL_PRINT_WIDGET	8

/*
 *	Following are bits for describing the annotation graphics.
 */

#define	GPL_OFF			(0x0)
#define	GPL_ON			(0x1)
#define	GPL_ON_LABEL		(0x2)
#define	GPL_ON_LABEL2		(0x4)
#define	GPL_ON_GRID		(0x8)
#define	GPL_ATTACHED		(0x10)
#define	GPL_ON_SCROLLBAR	(0x20)
#define	GPL_LEFT		(0x40)
#define	GPL_RIGHT		(0x80)

#define	GPL_IMMEDIATE		10

/*
 *	Following are return values from GPL_read_loop.
 */

#define	GPL_ERROR		0      /* Error return from select. */
#define	GPL_READ		1      /* OK to read from fdin. */
#define	GPL_TIMEOUT		2      /* Timeout occured. */

/*
 *	Following are variations on true, false.
 */

#define	GPL_TRUE		1
#define	GPL_FALSE		0
#define	GPL_ENABLE		1
#define	GPL_DISABLE		0
#define	GPL_PUT			1
#define	GPL_NOPUT		0

/*
 *	Following are official "null" values.
 */

#define	GPL_NULL_FLOAT		(float)(-1.234e-31)
#define	GPL_NULL_DOUBLE		(double)(-1.234e-31)
#define	GPL_NULL_INT		(int)(-123456789)

/*
 *	Following are GPL entry points.
 */

GatherPlot     *GPL_create (Display *, int, Visual *, int, Colormap, char *);
GatherPlot     *GPL_create_simple (Display *, char *, char *, int, int, int, int, int);

/*
 *	Following are GatherPlot attribute "hardwired" default value:
 */

#define	GPL_TRACE_PLOT_MODE_DEF	GPL_WIGGLE
#define	GPL_TRACE_TIME_RESIZE_DEF	GPL_FIT
#define	GPL_TRACE_UVAR_RESIZE_DEF	GPL_FIT
#define	GPL_GPL_WIN_X_FR_DEF	0.0
#define	GPL_GPL_WIN_Y_FR_DEF	0.0
#define	GPL_GPL_WIN_W_FR_DEF	1.0
#define	GPL_GPL_WIN_H_FR_DEF	1.0
#define	GPL_TRACE_PANE_X_FR_DEF	0.0
#define	GPL_TRACE_PANE_Y_FR_DEF	0.0
#define	GPL_TRACE_PANE_W_FR_DEF	0.0
#define	GPL_TRACE_PANE_H_FR_DEF	0.0
#define	GPL_TRACE_PANE_L_MARGIN_DEF	0
#define	GPL_TRACE_PANE_R_MARGIN_DEF	0
#define	GPL_TRACE_PANE_T_MARGIN_DEF	0
#define	GPL_TRACE_PANE_B_MARGIN_DEF	0
#define	GPL_T0_DEF		0.0
#define	GPL_TWIN_DEF		60.0
#define	GPL_UVAR0_DEF		1.0
#define	GPL_UVARWIN_DEF		-2.0
#define	GPL_AMP0_DEF		0.0
#define	GPL_AMPWIN_DEF		0.0
#define	GPL_TRACE_HEIGHT_DEF	0.9
#define	GPL_TRACE_BACK_HUE_DEF	0.0
#define	GPL_TRACE_BACK_LIT_DEF	0.9
#define	GPL_TRACE_BACK_SAT_DEF	0.2
#define	GPL_TRACE_FORE_HUE_DEF	0.0
#define	GPL_TRACE_FORE_LIT_DEF	0.3
#define	GPL_TRACE_FORE_SAT_DEF	0.8
#define	GPL_LABEL_FORE_HUE_DEF	0.0
#define	GPL_LABEL_FORE_LIT_DEF	0.0
#define	GPL_LABEL_FORE_SAT_DEF	0.0
#define	GPL_LABEL_BACK_HUE_DEF	240.0
#define	GPL_LABEL_BACK_LIT_DEF	0.9
#define	GPL_LABEL_BACK_SAT_DEF	0.2
#define	GPL_ARRIVAL_FORE_HUE_DEF	0.0
#define	GPL_ARRIVAL_FORE_LIT_DEF	0.0
#define	GPL_ARRIVAL_FORE_SAT_DEF	0.0
#define	GPL_ARRIVAL_BACK_HUE_DEF	240.0
#define	GPL_ARRIVAL_BACK_LIT_DEF	0.9
#define	GPL_ARRIVAL_BACK_SAT_DEF	0.2
#define	GPL_DEFINING_ARRIVAL_FORE_HUE_DEF	0.0
#define	GPL_DEFINING_ARRIVAL_FORE_LIT_DEF	0.0
#define	GPL_DEFINING_ARRIVAL_FORE_SAT_DEF	0.0
#define	GPL_DEFINING_ARRIVAL_BACK_HUE_DEF	120.0
#define	GPL_DEFINING_ARRIVAL_BACK_LIT_DEF	0.9
#define	GPL_DEFINING_ARRIVAL_BACK_SAT_DEF	0.2
#define	GPL_ASSOCIATED_ARRIVAL_FORE_HUE_DEF	0.0
#define	GPL_ASSOCIATED_ARRIVAL_FORE_LIT_DEF	0.0
#define	GPL_ASSOCIATED_ARRIVAL_FORE_SAT_DEF	0.0
#define	GPL_ASSOCIATED_ARRIVAL_BACK_HUE_DEF	60.0
#define	GPL_ASSOCIATED_ARRIVAL_BACK_LIT_DEF	0.9
#define	GPL_ASSOCIATED_ARRIVAL_BACK_SAT_DEF	0.2
#define	GPL_MAGNITUDE_ARRIVAL_FORE_HUE_DEF	0.0
#define	GPL_MAGNITUDE_ARRIVAL_FORE_LIT_DEF	0.0
#define	GPL_MAGNITUDE_ARRIVAL_FORE_SAT_DEF	0.0
#define	GPL_MAGNITUDE_ARRIVAL_BACK_HUE_DEF	300.0
#define	GPL_MAGNITUDE_ARRIVAL_BACK_LIT_DEF	0.9
#define	GPL_MAGNITUDE_ARRIVAL_BACK_SAT_DEF	0.2
#define	GPL_SEL_ARRIVAL_FORE_HUE_DEF	240.0
#define	GPL_SEL_ARRIVAL_FORE_LIT_DEF	0.9
#define	GPL_SEL_ARRIVAL_FORE_SAT_DEF	0.2
#define	GPL_SEL_ARRIVAL_BACK_HUE_DEF	0.0
#define	GPL_SEL_ARRIVAL_BACK_LIT_DEF	0.0
#define	GPL_SEL_ARRIVAL_BACK_SAT_DEF	0.0
#define	GPL_PRED_ARRIVAL_FORE_HUE_DEF	240.0
#define	GPL_PRED_ARRIVAL_FORE_LIT_DEF	0.9
#define	GPL_PRED_ARRIVAL_FORE_SAT_DEF	1.0
#define	GPL_DETECTION_FORE_HUE_DEF	0.0
#define	GPL_DETECTION_FORE_LIT_DEF	0.7
#define	GPL_DETECTION_FORE_SAT_DEF	1.0
#define	GPL_TIMEMARK_FORE_HUE_DEF	0.0
#define	GPL_TIMEMARK_FORE_LIT_DEF	1.0
#define	GPL_TIMEMARK_FORE_SAT_DEF	0.0
#define	GPL_TRACE_MOUSE_MODE_DEF	GPL_TIME_SCROLL
#define	GPL_TRACE_FONT_NAME_DEF		"-adobe-helvetica-bold-r-normal--12-120-75-75-p-70-iso8859-1"
#define	GPL_BACKUP_TRACE_FONT_NAME_DEF		"-adobe-helvetica-bold-r-normal--12-0-0-0-p-0-iso8859-1"
#define	GPL_TRACE_FONT_SMALL_NAME_DEF	"-adobe-helvetica-medium-r-normal--10-100-75-75-p-56-iso8859-1"
#define	GPL_BACKUP_TRACE_FONT_SMALL_NAME_DEF	"-adobe-helvetica-medium-r-normal--10-0-0-0-p-0-iso8859-1"
#define	GPL_AMP_SCALE_MODE_DEF	GPL_AUTO1
#define	GPL_TIME_FLAGS_DEF	(GPL_ON_LABEL | GPL_ON_GRID | GPL_ON_SCROLLBAR | GPL_ATTACHED)
#define	GPL_UVAR_FLAGS_DEF	(GPL_ON_LABEL | GPL_ON_GRID | GPL_ON_SCROLLBAR | GPL_ATTACHED)
#define	GPL_STA_FLAGS_DEF	(GPL_OFF)
#define	GPL_BUTTON_FLAGS_DEF	(GPL_ON | GPL_ATTACHED)
#define	GPL_TRACE_MOVEOUT_MODE_DEF	GPL_NONE
#define	GPL_TRACE_CLIPPING_FLAG_DEF	0
#define	GPL_TRACE_GAIN_DEF		1.0
#define	GPL_AMP_UVAR_MODE_DEF		GPL_OFF
#define	GPL_ARRIVAL_SELECT_DEF		GPL_ON
#define	GPL_ARRIVAL_EDIT_DEF		GPL_ON
#define	GPL_BUTTON_WIDTH_DEF		110
#define	GPL_ARRIVAL_MAG_WINDOW		20.0
#define	GPL_UVAR_ORDINAL_DEF		GPL_FALSE
#define	GPL_TRACE_MENU_WIDTH_DEF	80
#define	GPL_MAG_WIN_WIDTH_DEF		800
#define	GPL_MAG_WIN_HEIGHT_DEF		300
#define	GPL_SLN_WIN_WIDTH_DEF		0
#define	GPL_SLN_WIN_HEIGHT_DEF		0

typedef struct qpl_ {
	int npixels;
	int np;
	int istart;
	float *min;
	float *max;
	float *in;
	float *out;
	int *numb;
	int overlap;
	int noverlaps;
	struct qpl_ *overlaps;
} QPlot;


struct stalist_ {
	int index;
	int display;
	char label[64];
};

typedef struct gpl_image_ {
	int width;
	int height;
	int depth;
	int x;
	int y;
	int ctable[256];
	int red[256];
	int green[256];
	int blue[256];
	char *image;
} GPL_Image;

typedef struct overlay_ {	/* This is the Overlay structure that defines 
				   all of the state info for a single flag. */
	struct overlays_ *overlays_ptr;	/* Points to the associated Overlays
					   structure. */
	Display *display;		/* Destination display. */
	Window win;			/* Destination window. */
	GC gc;				/* Destination graphics context. */
	int o;				/* Overlay number. */
	int on;				/* Overlay on/off. */
	int clipped;			/* Is Overlay currently clipped? */
	int x;				/* Destination X offset. */
	int y;				/* Destination Y offset. */
	int w;				/* Overlay width. */
	int h;				/* Overlay height. */
	int d;				/* Overlay depth. */
	char *draw_ptr;			/* User pointer. */
	void (*draw_proc_ptr)();	/* User draw procedure. */
	void (*free_proc_ptr)();	/* User freeing procedure. */
	Pixmap back_pm;			/* Backing pixmap. */
	struct overlay_ *prev;		/* Previous flag in the linked list. */
	struct overlay_ *next;		/* Next flag in the linked list. */
} Overlay;

typedef struct overlays_ {	/* This is the Overlays structure that defines
				   a flags grouping. */
	Overlay *head;			/* Head of the Overlay linked list. */
	Overlay *tail;			/* Tail of the Overlay linked list. */
	int clip;			/* Destination clip flag */
	int x;				/* Destination clipping rectangle */
	int y;				
	int w;				
	int h;				
} Overlays;

#define	PASTEL	0

#define	X	0
#define	Y	1

#define	MODE_BLOCK	0
#define	MODE_BILIN	1

extern int postscript_plot ;
extern int gpl_show_fms;
extern int gpl_switch_arrival_buttons;

extern void GPL_get_Xdefaults ( GatherPlot *gpl, char *app_file, int *argc, char **argv );
extern void GPL_apply_Xdefaults ( GatherPlot *gpl );
extern GatherPlot * GPL_create ( Display *display, int screen, Visual *visual, int depth, Colormap colormap, char *progname );
extern _XFUNCPROTOEND GatherPlot * GPL_create_simple ( Display *display, char *title, char *progname, int x, int y, int w, int h, int vis_flag );
extern void GPL_draw ( GatherPlot *gpl );
extern void GPL_paint ( GatherPlot *gpl );
extern void GPL_paintall ( GatherPlot *gpl );
extern void GPL_paint_overlays ( GatherPlot *gpl );
extern void GPL_select_traces ( GatherPlot *gpl, char *stachan );
extern int stalist_cmp (const void *vptr1, const void *vptr2);
extern int GPL_event ( GatherPlot *gpl, XEvent *event );
extern void GPL_free ( GatherPlot *gpl );
extern void * GPL_get ( GatherPlot *gpl, ... );
extern int GPL_get_trace ( GatherPlot *gpl, void *trace_id, double t0, double twin, int npixels, int proc_flg, GPLTrace *gpl_trace );
extern int GPL_event_loop ( GatherPlot *gpl );
extern int GPL_read_loop ( GatherPlot *gpl, int fdin, double to );
extern int GPL_reply ( GatherPlot *gpl, char *reply );
extern int GPL_read ( GatherPlot *gpl, int fd, char *buf, int nbyte );
extern char * GPL_fgets ( GatherPlot *gpl, char *s, int n, FILE *stream );
extern void GPL_print ( GatherPlot *gpl, char *fname, char *cmd );
extern int redraw_arrival_glyph ( GatherPlot *gpl, double gtime, Overlay *olp, gpl_arrival_flag *arflg );
extern int redraw_tmark_glyph ( GatherPlot *gpl, double gtime, Overlay *olp, gpl_timemark_flag *tmflg );
extern int trace_selected ( GatherPlot *gpl, int trace_index );
extern int GPL_set ( GatherPlot *gpl, ... );
extern GatherPlot * gpl_find_root_gpl ( GatherPlot *gpl );
extern void gpl_set_arrival_phase ( GatherPlot *gpl, char *arrival_id, char *phase, char *fm, int put );
extern void gpl_set_arrival_phase_ovl ( GatherPlot *gpl, char *arrival_id, char *phase, char *fm, int put );
extern void gpl_set_arrival_fmauto ( GatherPlot *gpl, char *arrival_id, int put );
extern void gpl_set_arrival_fmauto_ovl ( GatherPlot *gpl, char *arrival_id, int put );
extern void gpl_set_arrival_delete ( GatherPlot *gpl, char *arrival_id, int put );
extern void gpl_set_arrival_delete_ovl ( GatherPlot *gpl, char *arrival_id, int put );
extern void gpl_set_arrival_deltim ( GatherPlot *gpl, char *arrival_id, double deltime, int put );
extern void gpl_set_arrival_time ( GatherPlot *gpl, char *arrival_id, double time, int put );
extern void gpl_set_tmark_time ( GatherPlot *gpl, int index, double time );
extern void gpl_set_arrival_time_ovl ( GatherPlot *gpl, char *arrival_id, double time, int put );
extern void gpl_set_tmark_time_ovl ( GatherPlot *gpl, int index, double time );
extern void gpl_set_arrival_deltim_ovl ( GatherPlot *gpl, char *arrival_id, double deltime, int put );
extern void gpl_set_arrival_add ( GatherPlot *gpl, char *trace_id, char *arrival_id, double time, char *phase, char *fm );
extern void gpl_set_tmark_add ( GatherPlot *gpl, double time, int index );
extern void gpl_set_tmark_delete ( GatherPlot *gpl, int index );
extern void gpl_set_arrival_add_ovl ( GatherPlot *gpl, char *trace_id, char *arrival_id, double time, char *phase, char *fm );
extern void gpl_set_tmark_add_ovl ( GatherPlot *gpl, double time, int index );
extern void gpl_set_tmark_delete_ovl ( GatherPlot *gpl, int index );
extern void gpl_set_arrival_select ( GatherPlot *gpl, char *arrival_id, int on );
extern void gpl_set_arrival_deltim_show ( GatherPlot *gpl, char *arrival_id, int on );
extern void gpl_set_arrival_select_ovl ( GatherPlot *gpl, char *arrival_id, int on );
extern void gpl_set_arrival_deltim_show_ovl ( GatherPlot *gpl, char *arrival_id, int on );
extern void gpl_draw_arflg_deltim ( GatherPlot *gpl, gpl_arrival_flag *arflg );
extern void gpl_set_uvar0 ( GatherPlot *gpl, float uvar0 );
extern void gpl_set_uvarwin ( GatherPlot *gpl, float uvarwin );
extern void gpl_set_uvar_both ( GatherPlot *gpl, float uvar0, float uvarwin );
extern void gpl_set_uvar0_full ( GatherPlot *gpl, float uvar0_full );
extern void gpl_set_uvarwin_full ( GatherPlot *gpl, float uvarwin_full );
extern void gpl_set_sta0 ( GatherPlot *gpl, int sta0 );
extern void gpl_set_stawin ( GatherPlot *gpl, int stawin );
extern void gpl_set_sta_both ( GatherPlot *gpl, int sta0, int stawin );
extern void gpl_set_sta_max ( GatherPlot *gpl, int max );
extern void gpl_set_sta0_full ( GatherPlot *gpl, int sta0_full );
extern void gpl_set_stawin_full ( GatherPlot *gpl, int stawin_full );
extern void gpl_set_sln ( GatherPlot *gpl, int i );
extern void gpl_set_arrival_ampper_show ( GatherPlot *gpl, char *arrival_id, int on );
extern void gpl_set_arrival_ampper_show_ovl ( GatherPlot *gpl, char *arrival_id, int on );
extern void gpl_draw_arflg_ampper ( GatherPlot *gpl, gpl_arrival_flag *arflg );
extern void gpl_set_arrival_ampper_auto ( GatherPlot *gpl, char *arrival_id, double time, int put );
extern void gpl_set_arrival_ampper_auto_ovl ( GatherPlot *gpl, char *arrival_id, double time, int put );
extern void gpl_set_ampper_ovl ( GatherPlot *gpl, char *arrival_id, double ap0amp, double ap1amp, double ap0time, double ap1time );
extern int gpl_auto_ampper ( GPLTrace *gpl_trace, double ftime, double twin_ap, double *ap0amp, double *ap1amp, double *ap0time, double *ap1time );
extern void gpl_draw ( GatherPlot *gpl );
extern void gpl_draw_arrivals ( GatherPlot *gpl );
extern void gpl_render_wiggle_plot ( GatherPlot *gpl, double tshift, double twini, int xnew, int wnew, int tplot_flg, int aplot_flg );
extern void gpl_draw_arrival ( Display *display, Window win, GC gc, int x, int y, int w, int h, gpl_arrival_flag *arflg );
extern void gpl_draw_tmark ( Display *display, Window win, GC gc, int x, int y, int w, int h, gpl_timemark_flag *tmflg );
extern void gpl_free_arrival ( Display *display, Window win, GC gc, gpl_arrival_flag *arflg );
extern void gpl_free_tmark ( Display *display, Window win, GC gc, gpl_timemark_flag *tmflg );
extern Pixmap gpl_create_arrival_pixmap ( GatherPlot *gpl, gpl_arrival_flag *arflg, int *w, int *h, int overlay, int selected );
extern void gpl_draw_times ( GatherPlot *gpl );
extern void gpl_draw_uvars ( GatherPlot *gpl );
extern void gpl_draw_stas ( GatherPlot *gpl );
extern void gpl_draw_moveout ( GatherPlot *gpl );
extern void gpl_paint ( GatherPlot *gpl );
extern void gpl_paint_subrgn ( GatherPlot *gpl, int x, int y, int w, int h );
extern void gpl_render_grayscale_plot ( GatherPlot *gpl, double tshift, double twini, int xnew, int wnew );
extern int gpl_get_uvar ( GatherPlot *gpl, int itrace, float *uvar );
extern int gpl_get_dt ( GatherPlot *gpl, int itrace, double *dt );
extern int gpl_get_sta ( GatherPlot *gpl, int itrace, char **sta, char **units );
extern int gpl_get_trace ( GatherPlot *gpl, int itrace, double t0, double twin, int npixels, int proc_flg, GPLTrace *gpl_trace );
extern int gpl_get_segment ( GatherPlot *gpl, int itrace, int iseg, double t0, double twin, int npixels, char **id_string, float *uvar, int *nsegs, double *tstart, double *dt, int *nsamps, float **data );
extern void gpl_draw_trace_rect ( GatherPlot *gpl, int itrace, double t1, double t2, float h1, float h2 );
extern int myXDrawColorContour ( Display *display, Drawable drawable, GC gc, char *colormap, int x, int y, int w, int h, double xl, double xr, double yb, double yt, double zmin, double zmax, double dz, char *mode, double x0, int nx, double dx, double y0, int ny, double dy, char *order, int stride, float *grid );
extern GPL_Image * create_image ( Display *display, int x, int y, int w, int h, int d, int cmap );
extern int interpolate_image ( GPL_Image *image, int nx, int ny, float *x, float *y, float *z, int order, int stride, double xmin, double xmax, double ymin, double ymax, double zmin, double dz, int mode );
extern int paint_image ( Display *display, Drawable drawable, GC gc, GPL_Image *image );
extern XPixel gpl_hls_to_pixel ( Display *display, Visual *visual, Colormap cmap, float h, float l, float s );
extern int gpl_event_gpl_win_parent ( GatherPlot *gpl, XEvent *event );
extern int gpl_event_expose ( GatherPlot *gpl, XExposeEvent *event );
extern int gpl_event_widget_pane ( GatherPlot *gpl, XEvent *event );
extern int gpl_event_sta_pane ( GatherPlot *gpl, XEvent *event );
extern int gpl_event_trace_pane ( GatherPlot *gpl, XEvent *event );
extern int gpl_event_time_pane ( GatherPlot *gpl, XEvent *event );
extern int gpl_event_uvar_pane ( GatherPlot *gpl, XEvent *event );
extern int gpl_event_status_pane ( GatherPlot *gpl, XEvent *event );
extern int gpl_event_time_scrl_pane ( GatherPlot *gpl, XEvent *event );
extern int gpl_event_uvar_scrl_pane ( GatherPlot *gpl, XEvent *event );
extern int gpl_event_sta_scrl_pane ( GatherPlot *gpl, XEvent *event );
extern int gpl_event_arrival ( GatherPlot *gpl, XButtonEvent *event, Overlay *olp );
extern int gpl_event_timemark ( GatherPlot *gpl, XButtonEvent *event, Overlay *olp );
extern int gpl_key_event_proc_def ( GatherPlot *gpl, XEvent *event, char *key, int nbytes );
extern int gpl_event_add_arrivals ( GatherPlot *gpl, XEvent *event );
extern int gpl_event_add_tmarks ( GatherPlot *gpl, XEvent *event );
extern int gpl_event_time_scroll ( GatherPlot *gpl, XEvent *event );
extern int gpl_event_time_zoom ( GatherPlot *gpl, XEvent *event );
extern int gpl_event_uvar_scroll ( GatherPlot *gpl, XEvent *event );
extern int gpl_event_uvar_zoom ( GatherPlot *gpl, XEvent *event );
extern int gpl_event_both_scroll ( GatherPlot *gpl, XEvent *event );
extern int gpl_event_both_zoom ( GatherPlot *gpl, XEvent *event );
extern int gpl_event_moveout ( GatherPlot *gpl, XEvent *event );
extern int gpl_locate_pane ( GatherPlot *gpl, XEvent *xevent );
extern int is_in_pane ( GPLPane *pane, int x, int y );
extern int locate_trace ( GatherPlot *gpl, XEvent *xevent, double *time );
extern int get_time ( GatherPlot *gpl, XEvent *xevent, double *time );
extern int locate_station ( GatherPlot *gpl, XEvent *xevent );
extern void gpl_draw_time_cursor ( Display *display, Window win, GC gc, int x, int y, int w, int h, GatherPlot *gpl );
extern int gpl_layout ( GatherPlot *gpl );
extern Box * Box_create ( Display *display, Window window, int line_mode );
extern void Box_free ( Box *box );
extern void Box_draw ( Box *box, int x, int y, int w, int h );
extern void Box_erase ( Box *box );
extern void gpl_ol_scale_menu_proc ( OlMenu *menu, int i );
extern int gpl_ol_quit_button_proc ( OlButton *button );
extern int gpl_ol_add_arrival_button_proc ( OlButton *button );
extern int gpl_ol_add_tmark_button_proc ( OlButton *button );
extern int gpl_ol_print_button_proc ( OlButton *button );
extern void gpl_ol_amp_menu_proc ( OlMenu *menu, int i );
extern void gpl_ol_mode_menu_proc ( OlMenu *menu, int i );
extern void gpl_ol_moveout_menu_proc ( OlMenu *menu, int i );
extern void gpl_ol_filter_menu_proc ( OlMenu *menu, int i );
extern void gpl_ol_trace_menu_proc ( OlMenu *menu, int i );
extern void gpl_ol_phase_menu_proc ( OlMenu *menu, int i );
extern void gpl_ol_fm_menu_proc ( OlMenu *menu, int i );
extern void gpl_ol_arrival_menu_proc ( OlMenu *menu, int i );
extern void gpl_ol_uvar_scroll_proc ( OlScrollbar *scrollbar, double view0, double view1, int immediate );
extern void gpl_ol_sta_scroll_proc ( OlScrollbar *scrollbar, double view0, double view1, int immediate );
extern void gpl_ol_time_scroll_proc ( OlScrollbar *scrollbar, double view0, double view1, int immediate );
extern Overlays * create_overlays ( void );
extern void set_clip_overlays ( Overlays *ovls, int x, int y, int w, int h );
extern Overlay * draw_overlay ( Overlays *overlays_ptr, Display *display, Window win, GC gc, int depth, int o, int on, int x, int y, int w, int h, void *draw_ptr, void (*draw_proc_ptr) ( ), void (*free_proc_ptr) ( ) );
extern void delete_overlay ( Overlay *overlay_ptr, int clrflg );
extern void delete_overlays ( Overlays *overlays_ptr, Window win, int o, int clrflg );
extern void turnoff_overlay ( Overlay *overlay_ptr );
extern void turnoff_overlays ( Overlays *overlays_ptr, int (*off_proc_ptr) ( ) );
extern void turnon_overlay ( Overlay *overlay_ptr );
extern void render_overlays ( Overlays *overlays_ptr );
extern void turnon_overlays ( Overlays *overlays_ptr, int (*on_proc_ptr) ( ) );
extern void destroy_overlays ( Overlays *overlays_ptr );
extern Overlay * locate_overlay ( Overlays *overlays_ptr, Window win, XEvent *xevent );
extern Overlay * locate_overlay_stack ( Overlays *overlays_ptr, Window win, XEvent *xevent, int index );
extern void move_overlay ( Overlay *overlay_ptr, int x, int y );
extern void move_resize_overlay ( Overlay *overlay_ptr, int x, int y, int w, int h );
extern QPlot * qpbin ( double ts, double dt, int nsamp, float *data, double tmin, double twin, int npixels, QPlot *qpl );
extern void qpfree ( QPlot *qpl );
extern void qpfreeoverlaps ( QPlot *qpl );
extern int qplot ( double ts, double dt, int nsamp, float *data, double tmin, double twin, int npixels, int x, int y, int h, float *ybot, float *ytop, int scale_type, float ygain, XPoint **xp );
extern int qplotsegs ( QPlot *qpl, int x, int y, int h, float *ybot, float *ytop, int scale_type, float ygain, Display *display, Drawable drawable, GC *gc, GC *gcov, void *gpl );
extern int nqplotsegs ( QPlot *qpl, float xdim, float ydim, float xlow, float ylow, float *ybot, float *ytop, int scale_type, int iclip, float ygain );
extern void get_nice_uvars ( float ustart, float uend, int maxincs, float *nstart, float *ninc );
extern void get_nice_fvals ( float fstart, float fend, int maxvals, int *istart, int *iend, float *ninc );
#endif

/* $Id$ */
